﻿Imports System.ComponentModel.DataAnnotations

Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core
Imports IdeaBlade.Validation

' Customer class with sample property interceptors.
Partial Public Class Customer

  ' Force the Country to upper case for display purposes.
  <AfterGet(EntityPropertyNames.Country)>
  Public Function UppercaseCountry(ByVal country As String) As String
    If Not String.IsNullOrEmpty(country) Then
      Return country.ToUpper()
    Else
      Return country
    End If
  End Function

  ' Log the set of any customer property.
  <AfterSet()>
  Public Sub AfterSetAnyProperty(ByVal args As IEntityPropertyInterceptorArgs)
    Dim msg As String = String.Format("Setting {0} to {1}", args.EntityProperty.Name, If(args.Value Is Nothing, "null", args.Value.ToString()))
    TraceLogger(msg)
  End Sub

  ' Force the CompanyName to upper case when set.
  <BeforeSet("CompanyName", Key:="C1")>
  Public Sub BeforeSetCompanyName(ByVal args As DataEntityPropertySetInterceptorArgs(Of Customer, String))
    If args.Value IsNot Nothing Then
      args.Value = args.Value.ToUpper()
    End If
  End Sub

  ' Log when CompanyName is set.
  <AfterSet("CompanyName", Key:="C2")>
  Public Sub AfterSetCompanyName(ByVal args As DataEntityPropertySetInterceptorArgs(Of Customer, String))
    Dim msg As String = String.Format("Set {0} to {1}", args.DataEntityProperty.Name, If(args.Value Is Nothing, "null", args.Value))
    TraceLogger(msg)
  End Sub


  Private Shared Sub TraceLogger(ByVal msg As String)
    Debug.WriteLine(msg)
  End Sub

End Class
