﻿#Region "usings"
Imports System.Collections
Imports System.Windows
Imports System.Windows.Controls

Imports IdeaBlade.EntityModel
Imports IdeaBlade.EntityModel.Extensions
Imports IdeaBlade.Core

Imports DomainModel
Imports IdeaBlade.Core.Reflection
Imports IdeaBlade.EntityModel.Security

#End Region

Namespace SilverlightPIApp

  Partial Public Class Page
    Inherits UserControl

#Region "ctor & load"

    Public Sub New()
      InitializeComponent()
      AddHandler Loaded, AddressOf Page_Loaded
    End Sub

    Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
      InitializePersistence()
      InitializeQueries()
    End Sub

    Private Sub InitializePersistence()
      DynamicInterceptors.AddDynamicEntityInterceptors()
      _entityManager = New NorthwindIBEntities(True)
      Login()
    End Sub

#End Region

#Region "Button Click Handlers"

    Private Sub btnSave_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Save()
    End Sub

#End Region

#Region "Initializequeries"

    Private Sub InitializeQueries()
      AddQueries()
      InitializeQueryCombo()
    End Sub

    Private Sub InitializeQueryCombo()

      _queryCombo.ItemsSource = _queries.Keys.ToList()
      _queryCombo.SelectedItem = _queries.Keys.FirstOrDefault()
      AddHandler _queryCombo.SelectionChanged, Sub() Fetch()
    End Sub

#End Region

#Region "Fetch"

    Private Async Sub Fetch()

      Dim query = GetQuery()
      If Nothing Is query Then
        WriteMessage("No query selected; pick a query.")
        Return
      End If

      WriteMessage("Fetching ...")

      Try
        Dim results = Await _entityManager.ExecuteQueryAsync(query)
        ' Special logic to handle the projection query - 
        '  can't bind to an anonymous type in Silverlight
        If AnonymousFns.IsAnonymousType(query.ElementType) Then
          _dataGrid.ItemsSource = DynamicTypeConverter.Convert(results)
        Else
          _dataGrid.ItemsSource = results
        End If

        ReportFetchCount(results)
      Catch ex As Exception
        WriteMessage(ex.Message)
      End Try
    End Sub

    Private Function GetQuery() As IEntityQuery
      Dim queryName = CStr(_queryCombo.SelectedItem)
      Return If(queryName Is Nothing, Nothing, _queries(queryName))
    End Function

    Private Sub ReportFetchCount(ByVal result As IEnumerable)
      WriteMessage(String.Format("Retrieved {0} customers", result.Cast(Of Object)().Count()))
    End Sub

#End Region

#Region "Queries "

    Private Sub AddQueries()

      _queries = New Dictionary(Of String, IEntityQuery)()

      _queries.Add("Get all Customers", _entityManager.Customers)

      _queries.Add("Get Customers starting with A", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("A")))

      _queries.Add("Get Customers with Orders in Oregon", _entityManager.Customers.Where(Function(c) c.Orders.Any(Function(o) o.ShipRegion = "OR")))

      _queries.Add("Get Customers starting with B from cache-only", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("B")).With(QueryStrategy.CacheOnly))

      _queries.Add("Get Customer info as projection", _entityManager.Customers.Select(Function(c) New With {Key c.CustomerID, Key c.CompanyName, Key c.City}))
    End Sub

#End Region

#Region "Login"

    Private Async Sub Login()
      Dim cred = New LoginCredential("demo", "demo", "demo")

      WriteMessage("Logging in ...")

      Try
        _entityManager.AuthenticationContext = Await Authenticator.Instance.LoginAsync(cred)

        WriteMessage("Logged in")

        _queryCombo.IsEnabled = True
        _btnSave.IsEnabled = True
        Fetch()
      Catch ex As Exception
        WriteMessage(ex.Message)
      End Try
    End Sub

#End Region

#Region "Save"

    Private Async Sub Save()

      If Not _entityManager.HasChanges() Then
        WriteMessage("No changes to save.")
        Return
      End If

      WriteMessage("Saving ...")

      Try
        Await _entityManager.SaveChangesAsync()
        WriteMessage("Changes saved")
      Catch ex As Exception
        WriteMessage(ex.Message)
      End Try

    End Sub
#End Region

#Region "Misc"

    Private Sub WriteMessage(ByVal msg As String)
      _txtStatus.Text = msg
    End Sub

#End Region

#Region "Fields"

    Private _entityManager As NorthwindIBEntities
    Private _queries As Dictionary(Of String, IEntityQuery)

#End Region
  End Class
End Namespace
